==> nnn-2024.2.17.tgz: package/h.js <==
import { is } from './is.js'

/**
 * The type of arguments of the `h` and `s` helpers.
 *
 * @typedef {Partial<Record<PropertyKey, any>> | null | undefined | Node | string | number | HArgs} HArgs1
 */

/**
 * The type of arguments of the `h` and `s` helpers.
 *
 * @typedef {[string | Node, ...HArgs1[]]} HArgs
 */

const NS = {
  xlink: 'http://www.w3.org/1999/xlink'
}

/**
 * @type {{
 *   (namespaceURI?: null | undefined): {
 *     <T extends keyof HTMLElementTagNameMap>(tag:  T, ...args1: HArgs1[]): HTMLElementTagNameMap[T];
 *     <N extends Node>                       (node: N, ...args1: HArgs1[]): N;
 *     (tagOrNode: string | Node, ...args1: HArgs1[]): Node;
 *   };
 *   (namespaceURI: 'http://www.w3.org/2000/svg'): {
 *     <T extends keyof SVGElementTagNameMap> (tag:  T, ...args1: HArgs1[]): SVGElementTagNameMap[T];
 *     <N extends Node>                       (node: N, ...args1: HArgs1[]): N;
 *     (tagOrNode: string | Node, ...args1: HArgs1[]): Node;
 *   };
 * }}
 */
const _h = (/** @type {string?=} */ namespaceURI) => {
  const createElement = namespaceURI == null
    ? (/** @type {string } */ tag) => document.createElement(tag)
    : (/** @type {string } */ tag) => document.createElementNS(namespaceURI, tag)

  const h = (/** @type {string | Node} */ tagOrNode, /** @type {HArgs1[]} */ ...args) => {
    const node = is(String, tagOrNode) ? createElement(tagOrNode) : tagOrNode

    args.forEach(arg => {
      let child = null

      if (arg instanceof Node) {
        child = arg
      } else if (is(String, arg) || is(Number, arg)) {
        // @ts-expect-error
        child = new Text(arg)
      } else if (is(Array, arg)) {
        // @ts-expect-error
        child = h(...arg)
      } else if (arg != null) {
        for (const name in arg) {
          const value = arg[name]

          if (name[0] === '$') {
            const name1 = name.slice(1)

            if (is(Object, value)) {
              // @ts-expect-error
              node[name1] = node[name1] ?? {}
              // @ts-expect-error
              Object.assign(node[name1], value)
            } else {
              // @ts-expect-error
              node[name1] = value
            }
          } else if (node instanceof Element) {
            const indexOfColon = name.indexOf(':')

            if (indexOfColon >= 0) {
              // @ts-expect-error
              const /** @type {string=} */ ns = NS[name.slice(0, indexOfColon)]

              if (ns != null) {
                const basename = name.slice(indexOfColon + 1)

                if (value === true) {
                  node.setAttributeNS(ns, basename, '')
                } else if (value === false) {
                  node.removeAttributeNS(ns, basename)
                } else {
                  node.setAttributeNS(ns, basename, is(String, value) ? value : '' + value)
                }
              }
            } else {
              if (value === true) {
                node.setAttribute(name, '')
              } else if (value === false) {
                node.removeAttribute(name)
              } else {
                node.setAttribute(name, is(String, value) ? value : '' + value)
              }
            }
          }
        }
      }

      if (child != null) {
        node.appendChild(child)
      }
    })

    return node
  }

  return h
}

/**
 * A lightweight [HyperScript](https://github.com/hyperhype/hyperscript)-style helper for creating and modifying `HTMLElement`s (see also `s`).
 *
 * - The first argument of type `string` specifies the tag of the element to be created.
 * - The first argument of type `Node` specifies the element to be modified.
 * - All other arguments of type `Partial<Record<PropertyKey, any>>` are mappings of attributes and properties.
 *   Keys starting with `$` specify *properties* (without the leading `$`) to be set on the element being created or modified.
 *   (Note that `$` is not a valid attribute name character.)
 *   All other keys specify *attributes* to be set by `setAttribute`.
 *   An attribute equal to `false` causes the attribute to be removed by `removeAttribute`.
 * - All other arguments of type `null` or `undefined` are simply ignored.
 * - All other arguments of type `Node` are appended to the element being created or modified.
 * - All other arguments of type `string`/`number` are converted to `Text` nodes and appended to the element being created or modified.
 * - All other arguments of type `HArgs` are passed to `h` and the results are appended to the element being created or modified.
 */
export const h = _h()

/**
 * A lightweight [HyperScript](https://github.com/hyperhype/hyperscript)-style helper for creating and modifying `SVGElement`s (see also `h`).
 *
 * - The first argument of type `string` specifies the tag of the element to be created.
 * - The first argument of type `Node` specifies the element to be modified.
 * - All other arguments of type `Partial<Record<PropertyKey, any>>` are mappings of attributes and properties.
 *   Keys starting with `$` specify *properties* (without the leading `$`) to be set on the element being created or modified.
 *   (Note that `$` is not a valid attribute name character.)
 *   All other keys specify *attributes* to be set by `setAttributeNS`.
 *   An attribute equal to `false` causes the attribute to be removed by `removeAttributeNS`.
 * - All other arguments of type `null` or `undefined` are simply ignored.
 * - All other arguments of type `Node` are appended to the element being created or modified.
 * - All other arguments of type `string`/`number` are converted to `Text` nodes and appended to the element being created or modified.
 * - All other arguments of type `HArgs` are passed to `s` and the results are appended to the element being created or modified.
 */
export const s = _h('http://www.w3.org/2000/svg')

export const tests = {
  h: (/** @type {import('bun:test').Expect} */ expect) => {
    const b = h('b')

    expect(b.outerHTML).toEqual('<b></b>')

    const i = h('i', 'text')

    h(b, i)

    expect(i.outerHTML).toEqual('<i>text</i>')
    expect(b.outerHTML).toEqual('<b><i>text</i></b>')

    h(i, { $className: 'some class' })

    expect(i.outerHTML).toEqual('<i class="some class">text</i>')
    expect(b.outerHTML).toEqual('<b><i class="some class">text</i></b>')
  },

  'h: innerText vs items': (/** @type {import('bun:test').Expect} */ expect) => {
    expect(h('span', 'text').outerHTML).toEqual('<span>text</span>')
    expect(h('span', { $innerText: 'text' }).outerHTML).toEqual('<span>text</span>')
  },

  'h: style': (/** @type {import('bun:test').Expect} */ expect) => {
    expect(h('div', { style: 'margin:0;padding:0' }).outerHTML)
      .toEqual('<div style="margin:0;padding:0"></div>')
    expect(h('div', { $style: 'margin:0;padding:0' }).outerHTML)
      .toEqual('<div style="margin: 0px; padding: 0px;"></div>')
    expect(h('div', { $style: { margin: 0, padding: 0 } }).outerHTML)
      .toEqual('<div style="margin: 0px; padding: 0px;"></div>')
  },

  'h: attributes vs properties': (/** @type {import('bun:test').Expect} */ expect) => {
    const input1 = h('input', { value: 42 })
    const input2 = h('input', { $value: '42' })

    expect(input1.value).toEqual('42')
    expect(input2.value).toEqual('42')

    expect(input1.outerHTML).toEqual('<input value="42">')
    expect(input2.outerHTML).toEqual('<input>')

    const checkbox1 = h('input', { type: 'checkbox', checked: true })
    const checkbox2 = h('input', { type: 'checkbox', $checked: true })

    expect(checkbox1.checked).toBe(true)
    expect(checkbox2.checked).toBe(true)

    expect(checkbox1.outerHTML).toEqual('<input type="checkbox" checked="">')
    expect(checkbox2.outerHTML).toEqual('<input type="checkbox">')
  },

  'h: nested properties': (/** @type {import('bun:test').Expect} */ expect) => {
    const div = h('div')

    // @ts-expect-error
    expect(div.key).toBeUndefined()

    h(div, { $key: { one: 1 } })

    // @ts-expect-error
    expect(div.key).toEqual({ one: 1 })

    h(div, { $key: { two: 2 } })

    // @ts-expect-error
    expect(div.key).toEqual({ one: 1, two: 2 })
  }
}

==> nnn-2024.2.20.tgz: package/h.js <==
import { is } from './is.js'

/**
 * The type of arguments of the `h` and `s` helpers.
 *
 * @typedef {Partial<Record<PropertyKey, any>> | null | undefined | Node | string | number | HArgs} HArgs1
 */

/**
 * The type of arguments of the `h` and `s` helpers.
 *
 * @typedef {[string | Node, ...HArgs1[]]} HArgs
 */

const NS = {
  xlink: 'http://www.w3.org/1999/xlink'
}

/**
 * @type {{
 *   (namespaceURI?: null | undefined): {
 *     <T extends keyof HTMLElementTagNameMap>(tag:  T, ...args1: HArgs1[]): HTMLElementTagNameMap[T];
 *     <N extends Node>                       (node: N, ...args1: HArgs1[]): N;
 *     (tagOrNode: string | Node, ...args1: HArgs1[]): Node;
 *   };
 *   (namespaceURI: 'http://www.w3.org/2000/svg'): {
 *     <T extends keyof SVGElementTagNameMap> (tag:  T, ...args1: HArgs1[]): SVGElementTagNameMap[T];
 *     <N extends Node>                       (node: N, ...args1: HArgs1[]): N;
 *     (tagOrNode: string | Node, ...args1: HArgs1[]): Node;
 *   };
 * }}
 */
const _h = (/** @type {string?=} */ namespaceURI) => {
  const createElement = namespaceURI == null
    ? (/** @type {string } */ tag) => document.createElement(tag)
    : (/** @type {string } */ tag) => document.createElementNS(namespaceURI, tag)

  const h = (/** @type {string | Node} */ tagOrNode, /** @type {HArgs1[]} */ ...args) => {
    const node = is(String, tagOrNode) ? createElement(tagOrNode) : tagOrNode

    args.forEach(arg => {
      let child = null

      if (arg instanceof Node) {
        child = arg
      } else if (is(String, arg) || is(Number, arg)) {
        // @ts-expect-error
        child = new Text(arg)
      } else if (is(Array, arg)) {
        // @ts-expect-error
        child = h(...arg)
      } else if (arg != null) {
        for (const name in arg) {
          const value = arg[name]

          if (name[0] === '$') {
            const name1 = name.slice(1)

            if (is(Object, value)) {
              // @ts-expect-error
              node[name1] = node[name1] ?? {}
              // @ts-expect-error
              Object.assign(node[name1], value)
            } else {
              // @ts-expect-error
              node[name1] = value
            }
          } else if (node instanceof Element) {
            const indexOfColon = name.indexOf(':')

            if (indexOfColon >= 0) {
              // @ts-expect-error
              const /** @type {string=} */ ns = NS[name.slice(0, indexOfColon)]

              if (ns != null) {
                const basename = name.slice(indexOfColon + 1)

                if (value === true) {
                  node.setAttributeNS(ns, basename, '')
                } else if (value === false) {
                  node.removeAttributeNS(ns, basename)
                } else {
                  node.setAttributeNS(ns, basename, is(String, value) ? value : '' + value)
                }
              }
            } else {
              if (value === true) {
                node.setAttribute(name, '')
              } else if (value === false) {
                node.removeAttribute(name)
              } else {
                node.setAttribute(name, is(String, value) ? value : '' + value)
              }
            }
          }
        }
      }

      if (child != null) {
        node.appendChild(child)
      }
    })

    return node
  }

  return h
}

/**
 * A lightweight [HyperScript](https://github.com/hyperhype/hyperscript)-style helper for creating and modifying `HTMLElement`s (see also `s`).
 *
 * - The first argument of type `string` specifies the tag of the element to be created.
 * - The first argument of type `Node` specifies the element to be modified.
 * - All other arguments of type `Partial<Record<PropertyKey, any>>` are mappings of attributes and properties.
 *   Keys starting with `$` specify *properties* (without the leading `$`) to be set on the element being created or modified.
 *   (Note that `$` is not a valid attribute name character.)
 *   All other keys specify *attributes* to be set by `setAttribute`.
 *   An attribute equal to `false` causes the attribute to be removed by `removeAttribute`.
 * - All other arguments of type `null` or `undefined` are simply ignored.
 * - All other arguments of type `Node` are appended to the element being created or modified.
 * - All other arguments of type `string`/`number` are converted to `Text` nodes and appended to the element being created or modified.
 * - All other arguments of type `HArgs` are passed to `h` and the results are appended to the element being created or modified.
 */
export const h = _h()

/**
 * A lightweight [HyperScript](https://github.com/hyperhype/hyperscript)-style helper for creating and modifying `SVGElement`s (see also `h`).
 *
 * - The first argument of type `string` specifies the tag of the element to be created.
 * - The first argument of type `Node` specifies the element to be modified.
 * - All other arguments of type `Partial<Record<PropertyKey, any>>` are mappings of attributes and properties.
 *   Keys starting with `$` specify *properties* (without the leading `$`) to be set on the element being created or modified.
 *   (Note that `$` is not a valid attribute name character.)
 *   All other keys specify *attributes* to be set by `setAttributeNS`.
 *   An attribute equal to `false` causes the attribute to be removed by `removeAttributeNS`.
 * - All other arguments of type `null` or `undefined` are simply ignored.
 * - All other arguments of type `Node` are appended to the element being created or modified.
 * - All other arguments of type `string`/`number` are converted to `Text` nodes and appended to the element being created or modified.
 * - All other arguments of type `HArgs` are passed to `s` and the results are appended to the element being created or modified.
 */
export const s = _h('http://www.w3.org/2000/svg')

export const tests = {
  h: (/** @type {import('chai').expect} */ expect) => {
    const b = h('b')

    expect(b.outerHTML).to.deep.equal('<b></b>')

    const i = h('i', 'text')

    h(b, i)

    expect(i.outerHTML).to.deep.equal('<i>text</i>')
    expect(b.outerHTML).to.deep.equal('<b><i>text</i></b>')

    h(i, { $className: 'some class' })

    expect(i.outerHTML).to.deep.equal('<i class="some class">text</i>')
    expect(b.outerHTML).to.deep.equal('<b><i class="some class">text</i></b>')
  },

  'h: innerText vs items': (/** @type {import('chai').expect} */ expect) => {
    expect(h('span', 'text').outerHTML).to.deep.equal('<span>text</span>')
    expect(h('span', { $innerText: 'text' }).outerHTML).to.deep.equal('<span>text</span>')
  },

  'h: style': (/** @type {import('chai').expect} */ expect) => {
    expect(h('div', { style: 'margin:0;padding:0' }).outerHTML)
      .to.deep.equal('<div style="margin:0;padding:0"></div>')
    expect(h('div', { $style: 'margin:0;padding:0' }).outerHTML)
      .to.deep.equal('<div style="margin: 0px; padding: 0px;"></div>')
    expect(h('div', { $style: { margin: 0, padding: 0 } }).outerHTML)
      .to.deep.equal('<div style="margin: 0px; padding: 0px;"></div>')
  },

  'h: attributes vs properties': (/** @type {import('chai').expect} */ expect) => {
    const input1 = h('input', { value: 42 })
    const input2 = h('input', { $value: '42' })

    expect(input1.value).to.deep.equal('42')
    expect(input2.value).to.deep.equal('42')

    expect(input1.outerHTML).to.deep.equal('<input value="42">')
    expect(input2.outerHTML).to.deep.equal('<input>')

    const checkbox1 = h('input', { type: 'checkbox', checked: true })
    const checkbox2 = h('input', { type: 'checkbox', $checked: true })

    expect(checkbox1.checked).to.be.true
    expect(checkbox2.checked).to.be.true

    expect(checkbox1.outerHTML).to.deep.equal('<input type="checkbox" checked="">')
    expect(checkbox2.outerHTML).to.deep.equal('<input type="checkbox">')
  },

  'h: nested properties': (/** @type {import('chai').expect} */ expect) => {
    const div = h('div')

    // @ts-expect-error
    expect(div.key).to.be.undefined

    h(div, { $key: { one: 1 } })

    // @ts-expect-error
    expect(div.key).to.deep.equal({ one: 1 })

    h(div, { $key: { two: 2 } })

    // @ts-expect-error
    expect(div.key).to.deep.equal({ one: 1, two: 2 })
  }
}
